@@ -3,6 +3,7 @@ from math import pi
 import tensorflow as tf
 
 from tf_fastmri_data.preprocessing_utils.fourier.cartesian import ortho_ifft2d
+from .crop import adjust_image_size
 
 
 @tf.function
@@ -48,23 +49,15 @@ def extract_smaps(kspace, low_freq_percentage=8):
 
 
 def non_cartesian_extract_smaps(kspace, trajs, dcomp, nufft_back, shape, low_freq_percentage=8):
-    def _crop_for_pad(image, shape, im_size):
-        to_pad = im_size[-1] - shape[0]
-        cropped_image = image[..., to_pad//2:-to_pad//2]
-        return cropped_image
     cutoff_freq = low_freq_percentage / 200 * tf.constant(pi)
     # Get the boolean mask for low frequency
     low_freq_bool_mask = tf.math.reduce_all(tf.math.less_equal(tf.abs(trajs[0]), cutoff_freq), axis=0)
     # Obtain the trajectory, kspace and density compensation for low frequency
     low_freq_traj = tf.boolean_mask(trajs, low_freq_bool_mask, axis=2)
     low_freq_kspace = tf.boolean_mask(kspace, low_freq_bool_mask, axis=2)
-    low_freq_dcomp = tf.boolean_mask(dcomp, low_freq_bool_mask, axis=1)
+    low_freq_dcomp = tf.boolean_mask(dcomp[:, None], low_freq_bool_mask, axis=2)
     coil_smap = nufft_back(low_freq_kspace * tf.cast(low_freq_dcomp, kspace.dtype), low_freq_traj)
-    coil_smap = tf.cond(
-            tf.math.greater_equal(shape, coil_smap.shape[-1]),
-            lambda: coil_smap,
-            lambda: _crop_for_pad(coil_smap, shape, coil_smap.shape),
-        )
-    low_freq_rss = tf.norm(coil_smap, axis=1)
+    coil_smap = adjust_image_size(coil_smap, shape, multicoil=True)
+    low_freq_rss = tf.norm(coil_smap, axis=1)[:, None]
     coil_smap = coil_smap / low_freq_rss
     return coil_smap
